#ifndef BOARD_VIEW_H
#define BOARD_VIEW_H 1

#include "config.h"

#include "pixmap_reader.h"
#include "enums.h"

extern unsigned char pn[14][8];

void wm_delete_window (Widget w, XtPointer client_data, XtPointer callData);
void ResizeEvent (Widget w, XtPointer client_data, XtPointer call_data);
void InputEvent (Widget w, XtPointer client_data, XtPointer call_data);
void ExposeEvent (Widget w, XtPointer client_data, XtPointer call_data);
void Left_Fire (Widget w, XtPointer client_data, XtPointer call_data);
void Left_Pass (Widget w, XtPointer client_data, XtPointer call_data);
void Right_Fire (Widget w, XtPointer client_data, XtPointer call_data);
void Right_Pass (Widget w, XtPointer client_data, XtPointer call_data);
/*void Unfire (Widget w, XtPointer client_data, XtPointer call_data);*/
void activate_Unfire (Widget w, XtPointer client_data, XtPointer call_data);
void disarm_Unfire (Widget w, XtPointer client_data, XtPointer call_data);
KeySym Key2Keysym (XKeyEvent *keyevent);
void CopyAsColor (Display *display,
		  Pixmap src, Drawable dest,
		  GC gc,
		  int src_x, int src_y,
		  unsigned int width, unsigned int height,
		  int dest_x, int dest_y,
		  unsigned long *bg_pixel,
		  unsigned long *fg_pixel);


#define D_color 3
#define D_highlighted 4
#define D_frozen 8
#define D_lazed 16


typedef struct _menu_arg
{
  id *ob;
  XtPointer arg;
} menu_arg;


typedef struct _menu_item
{
  char *name;
  XtCallbackProc proc;
  menu_arg *ma;
} menu_item;


Widget form_menu (Widget menu_bar, char *title, menu_item **items, int num);
void menu_bouncer (Widget w, XtPointer client_data, XtPointer call_data);



/************************************************************************/
/*                                                                      */
/************************************************************************/


@interface Board_View : Object
{
    XtAppContext app_context;
    Widget shell;
    Widget form;
    Widget frame;
    Widget draw;
    Atom WM_DELETE_WINDOW;

    Widget menu_bar;
    Widget game_cascade;
    menu_item *game_menu_items;

    Widget left_row_column;
    Widget left_fire;
    Widget left_pass;
    Widget left_draw;
    Widget right_row_column;
    Widget right_fire;
    Widget right_pass;
    Widget right_draw;

    
    Display *display;
    Visual *visual;
    Window window;
    Screen *screen;
    Colormap cmap;
    GC gc;
    int depth;

    XColor red;
    XColor green;
    XColor blue;
    XColor grey;
    XColor grey30;
    XColor white;
    XColor black;
    
    Pixmap pieces_pixmap;
    Pixmap back_store;
    Pixmap left_back_store;
    Pixmap right_back_store;

    id *lb;
    Player_Color board_color;

    /*Board_View_PS *bvps;
    Game_Finder *gf;*/

    id *bvps;
    id *gf;
}

- init_board : (int) argc
	     : (char **) argv
	     : (XtAppContext) ac
	     : (char *) display_name;
- (void) set_bvps : (id *) ps;
- (void) set_gf : (id *) new_gf;
- (void) delete_self;
- (void) set_logical_board : (id *) set_lb;
- (void) clear_board;
- (void) place_piece : (int) x : (int) y
		     : (int) kind_of_piece
		     : (int) direction
		     : (int) col;
- (void) clear_square : (int) x : (int) y;
- (void) place_beam : (int) x : (int) y : (int) direction;
- (void) remove_beam;
- (int) x_dir_add : (int) x : (int) dir;
- (int) y_dir_add : (int) y : (int) dir;
- (void) take_color : (Player_Color) c;
- (void) set_colors : (Player_Color) c;
- (void) update_moves : (int) red_moves : (int) green_moves;
- (void) send_message : (char *) message;
- (Display *) getDisplay;
- (void) menu_callback : (Widget) w
		       : (XtPointer) client_data
		       : (XtPointer) call_data;
- (menu_item *) new_menu_item : (char *) name
			      : (XtCallbackProc) proc
			      : (id *) ob
			      : (XtPointer) arg;
@end

extern void _XEditResCheckMessages();

#endif /* BOARD_VIEW_H */
