#include "board_view.h"
#include "logical_board.h"
#include "game_finder.h"
#include "board_view_ps.h"


#define GRID_SZ 32
#define PCS_SZ 28
#define DIF ((GRID_SZ-PCS_SZ)/2)


static int debug=0;

#if 0
static char * p_xpm[] = {
"4064 64 10 1",
" 	c #151505051717 g black  m black", /* black background */
".	c #F6F622221717 g grey50 m white", /* red in fleshy red */
"X	c #F6F622221717 g grey50 m black", /* #090971718585",  other color in fleshy red */
"o	c #34347C7C1717 g grey70 m white", /* fleshy green */
"O	c #343428282626 g grey20 m black", /* shadow of center teleporter */
"+	c #4C4C46464646 g black  m black", /* grey background square */
"#	c #E7E78E8E1717 g white  m black", /* mirrored red */
"$	c #5F5FFAFA1717 g white  m black", /* mirrored green */
"%	c #00000000EFEF g grey80 m white", /* the darker blue in center pieces */
"&	c #70707070EFEF g grey30 m white", /* the lighter blue in center pieces */
#endif



unsigned char pn[14][8] =
{
    {39,  39,  39,  39,  39,  39,  39,  39},  // 0 (blank) (b)
    {4,   5,   6,   7,   0,   1,   2,   3 },  // 1 (freezer) (f)
    {20,  21,  22,  23,  24,  25,  26,  27},  // 2 (part mirror stomper) (p)
    {40,  41,  42,  43,  44,  45,  46,  47},  // 3 (one-way) (o)
    {60,  61,  62,  63,  64,  65,  66,  67},  // 4 (mirror) (m)
    {80,  81,  82,  83,  84,  85,  86,  87},  // 5 (laser) (l)
    {100, 101, 102, 103, 104, 105, 106, 107}, // 6 (splitter) (s)
    {16,  36,  16,  36,  16,  36,  16,  36},  // 7 (bomb) (n)
    {56,  56,  56,  56,  56,  56,  56,  56},  // 8 (king) (k)
    {76,  76,  76,  76,  76,  76,  76,  76},  // 9 (teleporter) (t)
    {18,  18,  18,  18,  18,  18,  18,  18},  // 10 (mirror stomper) (x)
    {58,  58,  58,  58,  58,  58,  58,  58},  // 11 (telegate) (g)
    {120, 120, 120, 120, 120, 120, 120, 120}, // 12 (centerpit) (c)
    {143, 143, 143, 143, 143, 143, 143, 143}  // 13 (pit) (v)
    /* 135 */
};

char color_jump[ 14 ] = {0, 8, 8, 8, 8, 8, 8, 1, 1, 1, 1, 1, 0, 0};


int pc_x (int kind_of_piece,
	  int direction,
	  int color_of_piece)
{
  if (color_of_piece == C_green)
    {
      /* green */
      return (pn[ kind_of_piece ][ direction ] +
	      color_jump[ kind_of_piece ]) * PCS_SZ;
    }

  /* red or neither */
  return (pn[ kind_of_piece ][ direction ]) * PCS_SZ;
}




@implementation Board_View : Object

- init_board : (int) argc
	     : (char **) argv
	     : (XtAppContext) ac
	     : (char *) display_name
	   /*: (Player_Color) set_board_color*/
{
    [super init];


    app_context = ac;


    display = XtOpenDisplay (app_context,
			     display_name,
			     "PhaserChess",
			     "PhaserChess",
			     NULL, 0, &argc, argv);

    if (display == NULL)
	return NULL;
    

    /*********************************************************/

    WM_DELETE_WINDOW = XmInternAtom(display, "WM_DELETE_WINDOW", False);

    /* shell = XtAppCreateShell("PhaserChess", "PhaserChess",
			     applicationShellWidgetClass,
			     display, NULL, 0); */

    shell = XtVaAppCreateShell("PhaserChess",
			       "PhaserChess",
			       applicationShellWidgetClass,
			       display,
			       XmNdeleteResponse, XmDO_NOTHING,
			       XmNallowShellResize, False,
			       NULL);


    XmAddWMProtocolCallback(shell, WM_DELETE_WINDOW,
			    wm_delete_window, (XtPointer) self);
    XtAddEventHandler(shell, (EventMask)0, True,
		      (XtEventHandler)_XEditResCheckMessages, NULL);



    /*********************************************************/

    form = XtVaCreateWidget ("form",
			     xmFormWidgetClass,
			     shell,
			     NULL);


    /****************************menus**************************/
    {
      menu_item *gmi[ 4 ];
      gmi[ 0 ] = [self new_menu_item
		       : "start new game" : menu_bouncer
		       : (id *) self : (XtPointer) 0];
      gmi[ 1 ] = [self new_menu_item
		       : "query metaserver" : menu_bouncer
		       : (id *) self : (XtPointer) 1];
      gmi[ 2 ] = [self new_menu_item
		       : "search machine" : menu_bouncer
		       : (id *) self : (XtPointer) 2];
      gmi[ 3 ] = [self new_menu_item
		       : "play both colors" : menu_bouncer
		       : (id *) self : (XtPointer) 3];
      gmi[ 4 ] = [self new_menu_item
		       : "quit" : menu_bouncer
		       : (id *) self : (XtPointer) 4];


      menu_bar = XmCreateMenuBar (form, "menubar", NULL, 0);
      XtVaSetValues (menu_bar,
		     XmNleftAttachment, XmATTACH_FORM,
		     XmNtopAttachment, XmATTACH_FORM,
		     XmNrightAttachment, XmATTACH_FORM,
		     XmNbottomAttachment, XmATTACH_NONE, NULL);

      game_cascade = form_menu (menu_bar, "game", gmi, 5);

      XtManageChild (menu_bar);
    }
    /*************************************************************/

    board_color = C_neutral;


    /*
    if (board_color == C_red || board_color == C_both)
    {
    */
	left_row_column =  XtVaCreateWidget("left_row_column",
					    xmRowColumnWidgetClass,
					    form,
					    XmNleftAttachment, XmATTACH_FORM,
					    XmNtopAttachment, XmATTACH_WIDGET,
					    XmNtopWidget, menu_bar,
					    XmNbottomAttachment, XmATTACH_FORM,
					    NULL);


	left_fire = XtVaCreateWidget ("Fire",
				      xmPushButtonWidgetClass,
				      left_row_column,
				      NULL);

	left_pass = XtVaCreateWidget ("Pass",
				      xmPushButtonWidgetClass,
				      left_row_column,
				      NULL);

	left_draw = XtVaCreateWidget ("left_draw",
				      xmDrawingAreaWidgetClass,
				      left_row_column,
				      XmNheight, 150,
				      /*XmNwidth, GRID_SZ*15,
				      XmNtopAttachment, XmATTACH_FORM,*/
				      NULL);



	XtAddCallback(left_fire, XmNarmCallback, Left_Fire, (XtPointer) self);
	XtAddCallback(left_fire, XmNdisarmCallback,
		      disarm_Unfire, (XtPointer) self);
	/* XtAddCallback(left_fire, XmNactivateCallback,
		      activate_Unfire, (XtPointer) self); */
	XtAddCallback(left_pass, XmNactivateCallback,
		      Left_Pass, (XtPointer) self);
	XtAddCallback(left_draw, XmNexposeCallback,
		      ExposeEvent, (XtPointer) self);


	XtManageChild (left_fire);
	XtManageChild (left_pass);
	XtManageChild (left_draw);
	XtManageChild (left_row_column);
   /*
    }
    */

    /*********************************************************/

    draw = XtVaCreateWidget ("draw",
			     xmDrawingAreaWidgetClass,
			     form,
			     XmNwidth, GRID_SZ*15,
			     XmNheight, GRID_SZ*11,
			     XmNtopAttachment, XmATTACH_WIDGET,
			     XmNtopWidget, menu_bar,
			     NULL);

    /*
    if (board_color == C_red || board_color == C_both)
    {
    */
	XtVaSetValues (draw,
		       XmNleftAttachment, XmATTACH_WIDGET,
		       XmNleftWidget, left_row_column,
		       NULL);
   /*
    }
    */


    XtAddCallback(draw, XmNexposeCallback, ExposeEvent, (XtPointer) self);
    XtAddCallback(draw, XmNinputCallback, InputEvent, (XtPointer) self);
    XtAddCallback(draw, XmNresizeCallback, ResizeEvent, (XtPointer) self);

    XtManageChild (draw);

    /*********************************************************/


    /*
    if (board_color == C_green || board_color == C_both)
    {
    */
	right_row_column = XtVaCreateWidget("right_row_column",
					    xmRowColumnWidgetClass,
					    form,
					    XmNleftAttachment, XmATTACH_WIDGET,
					    XmNleftWidget, draw,
					    XmNtopAttachment, XmATTACH_WIDGET,
					    XmNtopWidget, menu_bar,
					    NULL );
	
	right_fire = XtVaCreateWidget("Fire",
				      xmPushButtonWidgetClass,
				      right_row_column,
				      NULL );

	right_pass = XtVaCreateWidget("Pass",
				      xmPushButtonWidgetClass,
				      right_row_column,
				      NULL );


	right_draw = XtVaCreateWidget ("right_draw",
				       xmDrawingAreaWidgetClass,
				       right_row_column,
				       XmNheight, 150,
				       /*XmNwidth, GRID_SZ*15,
					 XmNheight, GRID_SZ*11,
					 XmNtopAttachment, XmATTACH_FORM,*/
				       NULL);


	XtAddCallback(right_fire, XmNarmCallback,
		      Right_Fire, (XtPointer) self);
	XtAddCallback(right_fire, XmNdisarmCallback,
		      disarm_Unfire, (XtPointer) self);
	/* XtAddCallback(right_fire, XmNactivateCallback,
		      activate_Unfire, (XtPointer) self); */
	XtAddCallback(right_pass, XmNactivateCallback,
		      Right_Pass, (XtPointer) self);
	XtAddCallback(right_draw, XmNexposeCallback,
		      ExposeEvent, (XtPointer) self);


	XtManageChild (right_fire);
	XtManageChild (right_pass);
	XtManageChild (right_draw);
	XtManageChild (right_row_column);
   /*
    }
    */

    /*********************************************************/
    XtManageChild (form);
    /*********************************************************/


    XtRealizeWidget (shell);



    {
	Dimension width, height;

	XtVaGetValues (shell,
		       XmNwidth, &width,
		       XmNheight, &height,
		       NULL);

	if (debug)
	  printf ("width = %d, height = %d\n", width, height);

	XtVaSetValues (shell,
		       XmNminWidth, width,
		       XmNminHeight, height,
		       XmNmaxWidth, width,
		       XmNmaxHeight, height,
		       XmNbaseWidth, width,
		       XmNbaseHeight, height,
		       NULL);
    }



    /* display = XtDisplay (top_level); */
    visual = DefaultVisual (display, DefaultScreen (display));
    window = XtWindow (draw);
    screen = XtScreen (draw);
    gc = XCreateGC (display,
		    RootWindow (display, DefaultScreen (display)),
		    0, NULL);
    cmap = DefaultColormap (display, DefaultScreen(display));

    XtVaGetValues (shell, XtNdepth, &depth, NULL);

    if (debug)
      printf ("loading pixmap...\n");

    switch (visual->class)
    {
    case StaticGray:
    case GrayScale:
      if (depth == 1)
	pieces_pixmap = load_pixmap (display, window,
				     PIECES_XPM, 'm', "black");
      else
	pieces_pixmap = load_pixmap (display, window,
				     PIECES_XPM, 'g', "black");
      break;
    case StaticColor:
      if (depth == 1)
	pieces_pixmap = load_pixmap (display, window,
				     PIECES_XPM, 'm', "black");
      else
	pieces_pixmap = load_pixmap (display, window,
				     PIECES_XPM, 'c', "black");
	break;
    default:
      fprintf (stderr, "unknown visual class: %d\n", visual->class);
      /* no break ... fall through */
    case PseudoColor:
    case DirectColor:
    case TrueColor:
      pieces_pixmap = load_pixmap (display, window,
				   PIECES_XPM, 'c', "black");
      break;
    }
    if (debug)
      printf ("done.\n");


    back_store = XCreatePixmap (display, window,
				GRID_SZ*15, GRID_SZ*11, depth);
    XSetForeground (display, gc, black.pixel);
    XFillRectangle (display, back_store, gc, 0, 0, GRID_SZ*15, GRID_SZ*11);

    XParseColor (display, cmap, "red", &red);
    XAllocColor (display, cmap, &red);

    XParseColor (display, cmap, "green", &green);
    XAllocColor (display, cmap, &green);

    XParseColor (display, cmap, "blue", &blue);
    XAllocColor (display, cmap, &blue);

    XParseColor (display, cmap, "grey50", &grey);
    XAllocColor (display, cmap, &grey);

    XParseColor (display, cmap, "white", &white);
    XAllocColor (display, cmap, &white);

    XParseColor (display, cmap, "grey30", &grey30);
    XAllocColor (display, cmap, &grey30);

    XParseColor (display, cmap, "black", &black);
    XAllocColor (display, cmap, &black);



    {
	Dimension width, height;

	XtVaGetValues (left_draw,
		       XmNwidth, &width,
		       XmNheight, &height,
		       NULL);
	left_back_store = XCreatePixmap (display, window,
					 width, height, depth);
	XSetForeground (display, gc, grey30.pixel);
	XFillRectangle (display, left_back_store, gc, 0, 0, width, height);

	XtVaGetValues (right_draw,
		       XmNwidth, &width,
		       XmNheight, &height,
		       NULL);
	right_back_store = XCreatePixmap (display, window,
					  width, height, depth);
	XSetForeground (display, gc, grey30.pixel);
	XFillRectangle (display, right_back_store, gc, 0, 0, width, height);
    }


    [self clear_board];

    lb = NULL;
    bvps = NULL;
    gf = NULL;

    return self;
}


- (void) set_bvps : (id *) ps
{
  bvps = ps;
}


- (void) set_gf : (id *) new_gf
{
  gf = new_gf;
}



- (menu_item *) new_menu_item : (char *) name
			      : (XtCallbackProc) proc
			      : (id *) ob
			      : (XtPointer) arg
{
  menu_item *mi = (menu_item *) malloc (sizeof (menu_item));
  menu_arg *ma = (menu_arg *) malloc (sizeof (menu_arg));
  ma->ob = ob;
  ma->arg = arg;
  mi->name = name;
  mi->proc = proc;
  mi->ma = ma;
  return mi;
}


- (void) set_logical_board : (id *) set_lb
{
    lb = set_lb;
}



- (void) clear_square : (int) x : (int) y
{
    int bgc = (x + y) % 2;

    /* draw the background square */
    XSetForeground (display, gc, (bgc == 0) ? black.pixel : grey30.pixel);
    XFillRectangle (display, back_store, gc,
		    GRID_SZ*x+DIF, GRID_SZ*y+DIF, PCS_SZ, PCS_SZ);
    XFillRectangle (display, window, gc,
		    GRID_SZ*x+DIF, GRID_SZ*y+DIF, PCS_SZ, PCS_SZ);

    XFlush (display);
}


- (void) clear_board
{
    int x;
    int y;

    for (x=0; x<15; x++)
	for (y=0; y<11; y++)
	    [self clear_square : x : y];
}


- (void) refresh : (Drawable) d : (int) x : (int) y : (int) w : (int) h
{
  if (d == XtWindow (draw))
    XCopyArea (display, back_store, window,
	       gc, x, y, w, h, x, y);

  if (d == XtWindow (left_draw))
    XCopyArea (display, left_back_store, XtWindow (left_draw),
	       gc, x, y, w, h, x, y);

  if (d == XtWindow (right_draw))
    XCopyArea (display, right_back_store, XtWindow (right_draw),
	       gc, x, y, w, h, x, y);


  XFlush (display);
}


- (void) mouse_button_press : (int) x : (int) y : (int) n
{
    int bx = x/GRID_SZ;
    int by = y/GRID_SZ;

    if (bx < 0 || bx >= 15 || by < 0 || by >= 11)
	return;

    [(Logical_Board *) lb mouse_button_press : bx : by : n : board_color];
}


- (void) key_press : (int) key
{
    [(Logical_Board *) lb key_press : key];
}



- (void) draw_moves_border : (Widget) fore
			   : (Drawable) back
			   : (Pixel) c
			   : (int) this_board_controls
{
  Dimension width, height;
  int m = 5; /* margin */
  int x,y,w,h; /* box for dots */
  int d; /* diameter of dots */
  int i;

  XtVaGetValues (fore,
		 XmNwidth, &width,
		 XmNheight, &height,
		 NULL);

  w = (width*4)/5 - (2 * m);
  d = w - 2;
  h = (d+2) * 3;
  x = (width/2) - (w/2);
  y = m;

  if (this_board_controls) XSetForeground (display, gc, c);
  else XSetForeground (display, gc, black.pixel);

  for (i=1; i<=m; i++)
    XDrawRectangle (display, back, gc, x-i, y-i, w+2*i, h+2*i);
}


- (void) refresh_moves : (Widget) fore
		       : (Drawable) back
  		       : (int) moves
		       : (Pixel) c
		       : (int) this_board_controls
{
  Dimension width, height;
  int m = 5; /* margin */
  int x,y,w,h; /* box for dots */
  int d; /* diameter of dots */
  int i;

  XtVaGetValues (fore,
		 XmNwidth, &width,
		 XmNheight, &height,
		 NULL);

  XSetForeground (display, gc, grey30.pixel);
  XFillRectangle (display, back, gc, 0, 0, width, height);

  w = (width*4)/5 - (2 * m);
  d = w - 2;
  h = (d+2) * 3;
  x = (width/2) - (w/2);
  y = m;


  if (this_board_controls) XSetForeground (display, gc, c);
  else XSetForeground (display, gc, black.pixel);

  for (i=1; i<=m; i++)
    XDrawRectangle (display, back, gc, x-i, y-i, w+2*i, h+2*i);

  XSetForeground (display, gc, c);

  for (i=0; i<moves; i++)
    {
      XFillArc (display, back, gc,
		x+1, y+1 + (d+2) * i,
		d, d,
		0, 360 * 64);
    }

  XCopyArea (display, back, XtWindow (fore), gc, 0, 0, width, height, 0, 0);
}



- (void) place_piece : (int) x : (int) y
		     : (int) kind_of_piece
		     : (int) direction
		     : (int) col
{
    int bgc = (x + y) % 2;
    unsigned long bg, *fg;


    if (x < 0 || x >= 15)
    {
	fprintf (stderr, "place_piece: x out of range: %d\n", x);
	return;
    }

    if (y < 0 || y >= 11)
    {
	fprintf (stderr, "place_piece: y out of range: %d\n", y);
	return;
    }


    if (col & D_highlighted)
	bg = grey.pixel;
    else
	bg = ((bgc == 0) ? black.pixel : grey30.pixel);


    /* draw the background square */
    XSetForeground (display, gc, bg);
    XFillRectangle (display, back_store, gc, GRID_SZ*x+DIF, GRID_SZ*y+DIF, PCS_SZ, PCS_SZ);
    XFillRectangle (display, window, gc, GRID_SZ*x+DIF, GRID_SZ*y+DIF, PCS_SZ, PCS_SZ);


    fg = NULL;

    if (col & D_frozen)
	fg = &blue.pixel;

    if (col & D_lazed)
	fg = &white.pixel;


    CopyAsColor (display,
		 pieces_pixmap,
		 back_store,
		 gc,
		 pc_x (kind_of_piece, direction, col & D_color), 0,
		 PCS_SZ, PCS_SZ,
		 GRID_SZ * x + DIF, GRID_SZ * y + DIF,
		 &black.pixel,
		 fg);
    
    CopyAsColor (display,
		 pieces_pixmap,
		 window,
		 gc,
		 pc_x (kind_of_piece, direction, col & D_color), 0,
		 PCS_SZ, PCS_SZ,
		 GRID_SZ * x + DIF, GRID_SZ * y + DIF,
		 &black.pixel,
		 fg);
}
    



- (void) place_beam : (int) x : (int) y
		    : (int) direction
{
    int x1, x2, y1, y2;

    x1 = x * GRID_SZ + (GRID_SZ/2);
    y1 = y * GRID_SZ + (GRID_SZ/2);

    x2 = [self x_dir_add
	       : x
	       : direction] * GRID_SZ + (GRID_SZ/2);
    y2 = [self y_dir_add
	       : y
	       : direction] * GRID_SZ + (GRID_SZ/2);

    x2 = (x1+x2)/2;
    y2 = (y1+y2)/2;

    XSetLineAttributes (display, gc, 3, LineSolid, CapButt, JoinBevel);
    XSetForeground (display, gc, white.pixel);
    XDrawLine (display, window, gc, x1, y1, x2, y2);
    XFlush (display);
}


- (void) remove_beam
{
    XCopyArea (display,
	       back_store,
	       window,
	       gc,
	       0, 0, 15*GRID_SZ, 11*GRID_SZ,
	       0, 0);

    XFlush (display);
}


/*
- (void) process_message : (char *) message : (int) len
{
    switch (message[0])
    {
    case mt_clear_square:
	if (len != sizeof (clear_square_message))
	    fprintf (stderr, "size error, clear_square_message.\n");
	[self clear_square
	      : ((clear_square_message *)message)->x
	      : ((clear_square_message *)message)->y];
	break;
    case mt_place_piece:
	if (len != sizeof (place_piece_message))
	    fprintf (stderr, "size error, place_piece_message.\n");
	[self place_piece
	      : ((place_piece_message *)message)->x
	      : ((place_piece_message *)message)->y
	      : ((place_piece_message *)message)->kind_of_piece
	      : ((place_piece_message *)message)->direction
	      : ((place_piece_message *)message)->color_of_piece];
	break;
    case mt_clear_board:
	if (len != 1)
	    fprintf (stderr, "size error, clear_board_message.\n");
	[self clear_board];
	break;
    default:
	fprintf (stderr, "invalid message.\n");
	break;
    }
}    
*/


- (void) use_devil
{
    int lop;
    for (lop=0;lop<8;lop++)
    {
	pn[13][lop]=135;
    }
}


- (void) left_fire_event
{
    [(Logical_Board *) lb fire : C_red & board_color];
}

- (void) left_pass_event
{
    [(Logical_Board *) lb pass : C_red & board_color];
}

- (void) right_fire_event
{
    [(Logical_Board *) lb fire : C_green & board_color];
}

- (void) right_pass_event
{
    [(Logical_Board *) lb pass : C_green & board_color];
}

- (void) unfire
{
    [(Logical_Board *) lb unfire];
}


- (void) delete_self
{
  if (lb != NULL)
    [(Logical_Board *) lb delete_board_view : self];
  
  /* XtUnrealizeWidget (shell); */
  XtDestroyWidget (shell);

  /* free pixmaps, etc */

  [self free];
}


- (int) x_dir_add : (int) x : (int) dir
{
    switch (dir)
    {
    case 0:
    case 4:
	return x;
    case 1:
    case 2:
    case 3:
	return x+1;
    case 5:
    case 6:
    case 7:
	return x-1;
    default:
	fprintf (stderr, "bad direction in dir_add: %d\n", dir);
	return 0;
    }
}


- (int) y_dir_add : (int) y : (int) dir
{
    switch (dir)
    {
    case 2:
    case 6:
	return y;
    case 3:
    case 4:
    case 5:
	return y+1;
    case 7:
    case 0:
    case 1:
	return y-1;
    default:
	fprintf (stderr, "bad direction in dir_add: %d\n", dir);
	return 0;
    }
}

- (void) take_color : (Player_Color) c
{
  board_color |= c;
  [self draw_moves_border
	: left_draw
	: left_back_store
	: red.pixel
	: board_color & C_red];
  [self draw_moves_border
	: right_draw
	: right_back_store
	: green.pixel
	: board_color & C_green];
}


- (void) set_colors : (Player_Color) c
{
  board_color = c;
  [self draw_moves_border
	: left_draw
	: left_back_store
	: red.pixel
	: board_color & C_red];
  [self draw_moves_border
	: right_draw
	: right_back_store
	: green.pixel
	: board_color & C_green];
}


- (void) update_moves : (int) red_moves : (int) green_moves
{
  [self refresh_moves
	: left_draw
	: left_back_store
	: red_moves
	: red.pixel
	: board_color & C_red];
  [self refresh_moves
	: right_draw
	: right_back_store
	: green_moves
	: green.pixel
	: board_color & C_green];
}


- (void) send_message : (char *) message
{
  printf ("%s\n", message);
}

- (Display *) getDisplay
{
    return XtDisplay (shell);
}


- (void) menu_callback : (Widget) w
		       : (XtPointer) client_data
		       : (XtPointer) call_data
{
  int i = (int) client_data;

  switch (i)
    {
    case 0:
      [(Game_Finder *) gf new_game];
      break;
    case 1:
      [(Board_View_PS *) bvps ask_metaserver];
      break;
    case 2:
      [(Game_Finder *) gf search_machine];
      break;
    case 3:
      [(Logical_Board_PC *) lb request_color];
      break;
    case 4:
      exit (0);
    default:
      fprintf (stderr, "menu problem\n");
      break;
    }
}

@end


void wm_delete_window (Widget w, XtPointer client_data, XtPointer callData)
{
    [(Board_View *) client_data delete_self];
}



KeySym Key2Keysym (XKeyEvent *keyevent)
{
    XComposeStatus composestatus;
    KeySym keysym;
    char string[10];
    
    XLookupString(keyevent, string, 9, &keysym, &composestatus);
    return(keysym);
}


void Left_Fire (Widget w, XtPointer client_data, XtPointer call_data)
{
    [(Board_View *)client_data left_fire_event];
}

void Left_Pass (Widget w, XtPointer client_data, XtPointer call_data)
{
    [(Board_View *)client_data left_pass_event];
}

void Right_Fire (Widget w, XtPointer client_data, XtPointer call_data)
{
    [(Board_View *)client_data right_fire_event];
}

void Right_Pass (Widget w, XtPointer client_data, XtPointer call_data)
{
    [(Board_View *)client_data right_pass_event];
}


void disarm_Unfire (Widget w, XtPointer client_data, XtPointer call_data)
{
    [(Board_View *)client_data unfire];
}


/*
void activate_Unfire (Widget w, XtPointer client_data, XtPointer call_data)
{
    printf ("activate unfire.\n");
    [(Board_View *)client_data unfire];
}
*/


void ExposeEvent (Widget w, XtPointer client_data, XtPointer call_data)
{
  XEvent *event = ((XmDrawingAreaCallbackStruct *)call_data)->event;

  [(Board_View *)client_data refresh
		 : event->xexpose.window
		 : event->xexpose.x
		 : event->xexpose.y
		 : event->xexpose.width
		 : event->xexpose.height];
}


void InputEvent (Widget w, XtPointer client_data, XtPointer call_data)
{
    KeySym keysym;
    int btn_num;

    XEvent *event = ((XmDrawingAreaCallbackStruct *)call_data)->event;

    switch(event->type)
    {
    case ButtonPress:
	switch(event->xbutton.button)
	{
	case Button1:
	    btn_num = 0;
	    break;
	case Button2:
	    btn_num = 1;
	    break;
	case Button3:
	    btn_num = 2;
	    break;
	default:
	    break;
	}

	[(Board_View *) client_data mouse_button_press
			: event->xbutton.x
			: event->xbutton.y
			: btn_num];
	break;
    case KeyPress:
	keysym = Key2Keysym ((XKeyEvent *) (event));
	[(Board_View *) client_data key_press : keysym];
	break;
    default:
	break;
    }
}


void ResizeEvent (Widget w, XtPointer client_data, XtPointer call_data)
{
    /* XEvent *event = ((XmDrawingAreaCallbackStruct *)call_data)->event; */
}


/* pick the upper left dot, use its color as
   the background color.  Every pixel that
   is not the same color as the background
   color, draw as col, unless col is null.
   if col is null, draw every dot as
   its own color.
*/

void CopyAsColor (Display *display,
		  Pixmap src, Drawable dest,
		  GC gc,
		  int src_x, int src_y,
		  unsigned int width, unsigned int height,
		  int dest_x, int dest_y,
		  unsigned long *bg_pixel,
		  unsigned long *fg_pixel)
{
    int x, y, p;
    XImage *i;

    int corner_p;

    if (fg_pixel != NULL)
	XSetForeground (display, gc, *fg_pixel);

    i = XGetImage (display, src, src_x, src_y, width, height, ~0, ZPixmap);

    if (i==NULL)
    {
	fprintf (stderr, "XGetImage failed\n");
	return;
    }

    if (bg_pixel == NULL)
	corner_p = XGetPixel(i,0,0);
    else
	corner_p = (*bg_pixel);

    width+=src_x;
    height+=src_y;

    for (x=src_x; x<width; x++)
    {
	for (y=src_y; y<height; y++)
	{
	    p = XGetPixel(i,x-src_x,y-src_y);
	    if (p != corner_p)
	    {
		if (fg_pixel == NULL)
		    XSetForeground (display, gc, p);
		XDrawPoint(display, dest, gc, x-src_x+dest_x, y-src_y+dest_y);
	    }
	}
    }

    XFlush(display);
}


Widget form_menu (Widget menu_bar, char *title, menu_item **items, int num)
{
  Widget pulldown, cascade;
  XmString title_str;
  int i;

  pulldown = XmCreatePulldownMenu (menu_bar, "menu", NULL, 0);
  cascade = XtCreateWidget (title, xmCascadeButtonWidgetClass,
			    menu_bar, NULL, 0);

  title_str = XmStringCreateLocalized (title);
  XtVaSetValues (cascade,
		 XmNsubMenuId, pulldown,
		 XmNlabelString, title_str,
		 NULL, NULL);

  XmStringFree (title_str);
  XtManageChild (cascade);

  for (i=0; i<num; i++)
    {
      Widget pb;

      pb = XtCreateWidget (items[ i ]->name,
			   xmPushButtonWidgetClass,
			   pulldown,
			   NULL, 0);
      XtManageChild (pb);
      XtAddCallback (pb,
		     XmNactivateCallback,
		     items[ i ]->proc,
		     (XtPointer) items[ i ]->ma);
    }

  return cascade;
}


void menu_bouncer (Widget w, XtPointer client_data, XtPointer call_data)
{
  menu_arg *ma = (menu_arg *) client_data;
  Board_View *bv = (Board_View *) ma->ob;
  [bv menu_callback : w : ma->arg : call_data];
}
