#include "config.h"
#include "game_finder.h"
#include "logical_board.h"
#include "logical_board_ps.h"

void wm_delete_window (Widget w, XtPointer client_data, XtPointer callData);
void call_world_think (XtPointer xtp, int *ip, XtInputId *xtii);

static int debug=0;

#ifdef NEED_GETHOSTNAME_PROTO
int gethostname(char *name, int namelen);
#endif


@implementation Game_Finder : Object

- init : (int) argc
       : (char **) argv
       : (XtAppContext) ac
       : (char *) display_name
       : (Board_View *) set_bv
       : (Board_View_PS *) set_bvps
       : (char *) set_server_machine
       : (int) set_server_port
       : (int) set_listen_port
{
    Arg list_arg[ 1 ];

    [super init];

    app_context = ac;
    bv = set_bv;
    bvps = set_bvps;
    server_machine = set_server_machine;
    server_port = set_server_port;
    listen_port = set_listen_port;
    lbpc = NULL;

    [bvps set_game_finder : (id *) self];

    games = [[Vector alloc] init];

    search_machine_str = "Search Machine";
    ask_metaserver_str = "Ask MetaServer";
    new_game_str = "Start New Game";
    other_color_str = "Request Control of Another Color";
    kill_server_str = "Ask Server to Exit";
    quit_str = "Quit";


    display = XtOpenDisplay (app_context,
			     display_name,
			     "PhaserChess",
			     "PhaserChess",
			     NULL, 0, &argc, argv);

    if (display == NULL)
	return NULL;

    WM_DELETE_WINDOW = XmInternAtom (display, "WM_DELETE_WINDOW", False);

    shell = XtVaAppCreateShell ("PhaserChess",
				"PhaserChess",
				applicationShellWidgetClass,
				display,
				XmNdeleteResponse, XmDO_NOTHING,
				XmNallowShellResize, False,
				NULL);

    XmAddWMProtocolCallback (shell, WM_DELETE_WINDOW,
			     wm_delete_window, (XtPointer) self);

    XtAddEventHandler (shell, (EventMask) 0, True,
		       (XtEventHandler) _XEditResCheckMessages, NULL);


    row_column =  XtVaCreateWidget ("row_column",
				    xmRowColumnWidgetClass,
				    shell,
				    NULL);

    machine_name = XtVaCreateManagedWidget ("machine name",
					    xmTextFieldWidgetClass,
					    row_column,
					    XmNcolumns, 40,
					    NULL); 

    /*
    search_machine_button = XtVaCreateWidget (search_machine_str,
					      xmPushButtonWidgetClass,
					      row_column, NULL);
    ask_metaserver_button = XtVaCreateWidget (ask_metaserver_str,
					      xmPushButtonWidgetClass,
					      row_column, NULL);
    new_game_button = XtVaCreateWidget (new_game_str,
					xmPushButtonWidgetClass,
					row_column, NULL);
    other_color_button = XtVaCreateWidget (other_color_str,
					   xmPushButtonWidgetClass,
					   row_column, NULL);
    kill_server_button = XtVaCreateWidget (kill_server_str,
					   xmPushButtonWidgetClass,
					   row_column, NULL);
					   */


    XtSetArg (list_arg[ 0 ], XmNlistSizePolicy, XmCONSTANT);
    game_list = XmCreateScrolledList (row_column, "game list", list_arg, 1); 

    XtVaSetValues(game_list, 
		  xmListWidgetClass, row_column,
		  XmNselectionPolicy, XmBROWSE_SELECT,
		  XmNvisibleItemCount, 7,
		  XmNscrollBarDisplayPolicy, XmSTATIC,
		  NULL);


    /*
    quit_button = XtVaCreateWidget (quit_str,
				    xmPushButtonWidgetClass,
				    row_column, NULL);
				    */


    XtAddCallback (machine_name, XmNactivateCallback,
		   TF_Callback, (XtPointer) self);

    /*
    XtAddCallback (search_machine_button, XmNactivateCallback,
		   BP_Callback, (XtPointer) self);
    XtAddCallback (ask_metaserver_button, XmNactivateCallback,
		   BP_Callback, (XtPointer) self);
    XtAddCallback (new_game_button, XmNactivateCallback,
		   BP_Callback, (XtPointer) self);
    XtAddCallback (other_color_button, XmNactivateCallback,
		   BP_Callback, (XtPointer) self);
    XtAddCallback (kill_server_button, XmNactivateCallback,
		   BP_Callback, (XtPointer) self);
		   */
    XtAddCallback (game_list, XmNdefaultActionCallback,
		   List_Callback, (XtPointer) self);
    XtAddCallback (game_list, XmNbrowseSelectionCallback,
		   List_Callback, (XtPointer) self);
    /*
    XtAddCallback (quit_button, XmNactivateCallback,
		   BP_Callback, (XtPointer) self);
		   */


    XtManageChild (machine_name);
    /*
    XtManageChild (search_machine_button);
    XtManageChild (ask_metaserver_button);
    XtManageChild (new_game_button);
    XtManageChild (other_color_button);
    XtManageChild (kill_server_button);
    */
    XtManageChild (game_list);
    /*
    XtManageChild (quit_button);
    */
    XtManageChild (row_column);

    XtRealizeWidget (shell);

    {
	Dimension width, height;

	XtVaGetValues (shell,
		       XmNwidth, &width,
		       XmNheight, &height,
		       NULL);

	if (debug)
	  printf ("shell width = %d, height = %d\n", width, height);

	XtVaSetValues (shell,
		       XmNminWidth, width,
		       XmNminHeight, height,
		       XmNmaxWidth, width,
		       XmNmaxHeight, height,
		       XmNbaseWidth, width,
		       XmNbaseHeight, height,
		       NULL);
    }


    XmTextFieldSetString (machine_name, server_machine);

    [self search_machine];

    return self;
}


- (void) forget_game_list
{
  XmListDeleteAllItems (game_list);
  while ([games size] > 0)
    delete_game_info ((game_info *) [games delete : 0]);
}


- (void) fill_game_list
{
  int i;
  XmString xmstr;


  XmListDeleteAllItems (game_list);

  for (i=0; i<[games size]; i++)
    {
      game_info *gi = (game_info *) [games get : i];
      if (gi == NULL) continue;
      xmstr = [self game_info_to_XmString : gi];
      XmListAddItemUnselected (game_list, xmstr, 0);
      XmStringFree(xmstr);
    }
}



- (int) find_index_of_game : (game_info *) find_gi
{
  int i;

  for (i=0; i<[games size]; i++)
    {
      game_info *gi = (game_info *) [games get : i];
      if (gi != NULL)
	if ((strcmp (gi->ip, find_gi->ip) == 0) &&
	    (gi->port == find_gi->port))
	  {
	    if (debug)
	      printf ("find_index_of_game found game at index %d\n", i);
	    return i;
	  }
    }
  
  return -1;
}



- (void) append_to_game_list : (game_info *) gi
{
  XmString xmstr = [self game_info_to_XmString : gi];
  int i;

  if (debug)
    printf ("game_info: %s@%s.%s:%d, %d players\n",
	    gi->uname,
	    gi->host,
	    gi->domain,
	    gi->port,
	    gi->num_players);

  i = [self find_index_of_game : gi];
  if (i < 0)
    {
      [games add : (unsigned char *) gi];
      XmListAddItemUnselected (game_list, xmstr, 0);
      XmStringFree(xmstr);
    }
  else
    {
      [games set : i : (unsigned char *) gi];
      [self fill_game_list];
    }
}


- (XmString) game_info_to_XmString : (game_info *) gi
{
  /* xxx@yyy.zzz, %d players */
  XmString xmstr;
  char *ts = (char *) malloc (strlen( gi->uname ) +
			      strlen( gi->host ) +
			      strlen( gi->domain ) +
			      14);
  char n[ 2 ] = {'\0', '\0'};
  
  strcpy (ts, gi->uname);
  strcat (ts, "@");
  strcat (ts, gi->host);
  if (strlen (gi->domain) > 0)
    {
      strcat (ts, ".");
      strcat (ts, gi->domain);
    }
  strcat (ts, ", ");
  n[ 0 ] = gi->num_players + '0';
  strcat (ts, n);
  strcat (ts, " players");

  xmstr = XmStringCreateSimple (ts);

  return xmstr;
}



- (void) delete_self
{
  XtPopdown (shell);
}


- (void) bp_callback : (Widget) w : (XtPointer) call_data
{
  /*XEvent *event = ((XmPushButtonCallbackStruct *) call_data)->event;*/

  XmString    xms = NULL;
  char        *s = NULL;

  if (debug)
    printf ("in push button callback\n");

  XtVaGetValues(w, XmNlabelString, &xms, NULL);
  if (xms == NULL)
    {
      fprintf(stderr, "XmString was NULL\n");
      return;
  }

  XmStringGetLtoR(xms, XmSTRING_DEFAULT_CHARSET, &s);
  if (s == NULL)
    {
      fprintf(stderr, "Conversion to String failed\n");
      return;
    }

  XmStringFree(xms);

  if (strcmp (s, search_machine_str) == 0)
    [self search_machine];
  else if (strcmp (s, ask_metaserver_str) == 0)
    [bvps ask_metaserver];
  else if (strcmp (s, new_game_str) == 0)
    [self new_game];
  else if (strcmp (s, other_color_str) == 0)
    [lbpc request_color];
  else if (strcmp (s, kill_server_str) == 0)
    [lbpc kill_server];
  else if (strcmp (s, quit_str) == 0)
    exit (0);
  else
    fprintf (stderr, "button problem in gamefinder.m\n");

  XtFree(s);
}


- (void) connect_to_game : (char *) host
			 : (char *) domain
			 : (char *) ip
			 : (int) port
{
  char hostname[ 256 ];
  char *remote_host;
  int rhl; /* remote host length */

  rhl = strlen (host) + strlen (domain) + 2;
  remote_host = (char *) malloc (rhl);
  strcpy (remote_host, host);
  if (strlen (domain) > 0)
    {
      strcat (remote_host, ".");
      strcat (remote_host, domain);
    }

  if (lbpc != NULL)
    {
      /* need to close connection to current server */
      XtRemoveInput (lbpc_XtInputId);
      [lbpc close_channel];
      [lbpc free];
      lbpc = NULL;
      [bv set_colors : C_neutral];
    }


  if (gethostname (hostname, 256) < 0)
    {
      perror ("gethostname");
      fprintf (stderr, "gethostname failed, trying with \"localhost\"\n");
      strcpy (hostname, "localhost");
    }

  lbpc = [[Logical_Board_PC alloc] init
				   : bvps
				   /*: remote_host*/
				   : ip
				   : port];

  if (lbpc == NULL)
    {
      fprintf (stderr, "couldn't reach server: %s:%d (%s)\n",
	       remote_host, port, ip);
      free (remote_host);
      return;
    }

  if  (strcmp (server_machine, "localhost") == 0)
    strcpy (hostname, "localhost");

  [lbpc add_board_view : hostname : listen_port];
  [bv set_logical_board : (id *) lbpc];

  [lbpc request_color];

  /* data comes back on that channel as well, so... */
  lbpc_XtInputId = XtAppAddInput (app_context,
				  [bvps get_stream_file_d : [lbpc get_ch]],
				  (XtPointer) XtInputReadMask,
				  call_world_think,
				  NULL);

  free (remote_host);
}


- (void) start_server
{
  Logical_Board *lb;
  Logical_Board_PS *lbps = NULL;


  if (debug)
    printf ("::releaseing X display...\n");

  /* let go of the X windows... */
  close (ConnectionNumber (XtDisplay (shell)));
  close (ConnectionNumber ([bv getDisplay]));

  /* let go of the controlling terminal... */
  if (!debug)
    close_controlling_terminal ();

  if (debug)
    printf ("::init logical board...\n");

  lb = [[Logical_Board alloc] init_logical_board];
  [lb load_file : STARTUP_PCS];


  if (debug)
    printf ("::init logical board ps\n");

  while (lbps == NULL)
    {
      lbps = [[Logical_Board_PS alloc] init_ps : server_port : lb];
      if (lbps == NULL) server_port ++;
    }


  if (debug)
    printf ("::looping...\n");


  while (1)
    {
      [lbps catch : 20L : 0L];
      [lbps tell_details_to_ms]; /* every 20 seconds, so
				    the ms doesn't decide
				    that this server is dead */
    }
}


- (void) new_game
{
  int count=0;
  int current_games = 0;

  if (games != NULL)
    current_games = [games size];

  if (debug)
    printf ("in new_game\n");

  switch (fork ())
    {
    case -1:
      perror ("fork");
      exit (1);
    case 0:
      if (debug)
	printf ("::fork done... starting server.\n");
      [self start_server];
    default:
      break;
    }

  do
    {
      sleep (1);
      [self search_machine];
      count ++;
      if (count == 10) break; /* no server???! */
    }
  while ([games size] == current_games);
}


- (void) search_machine
{
  char *value = XmTextFieldGetString (machine_name);

  if (debug)
    printf ("in tf_callback: %s\n", value);

  while ([games size] > 0)
    delete_game_info ((game_info *) [games delete : 0]);
    
  games = [bvps search_machine_for_servers : value : server_port];

  [self fill_game_list];

  XtFree (value);
}


- (void) tf_callback : (Widget) w : (XtPointer) call_data
{
  [self search_machine];
}


- (void) list_callback : (Widget) w : (XtPointer) call_data
{
  XmListCallbackStruct *cbs = (XmListCallbackStruct *)call_data;
  int pos;

  if (debug)
    printf ("in list callback\n");

  /*if (cbs->reason != XmCR_DEFAULT_ACTION)*/
  if (cbs->reason != XmCR_BROWSE_SELECT)
    return;

  pos = cbs->item_position - 1;

  /*
  if (games != NULL)
    {
      printf ("user='%s', ", games->gis[ pos ]->uname);
      printf ("machine='%s', ", games->gis[ pos ]->host);
      printf ("port=%d\n", games->gis[ pos ]->port);
    }
    */

  [self connect_to_game
	: ((game_info *) [games get : pos])->host
	: ((game_info *) [games get : pos])->domain
	: ((game_info *) [games get : pos])->ip
	: ((game_info *) [games get : pos])->port];
}

@end


void BP_Callback (Widget w, XtPointer client_data, XtPointer call_data)
{
    [(Game_Finder *) client_data bp_callback : w : call_data];
}

void TF_Callback (Widget w, XtPointer client_data, XtPointer call_data)
{
    [(Game_Finder *) client_data tf_callback : w : call_data];
}

void List_Callback (Widget w, XtPointer client_data, XtPointer call_data)
{
    [(Game_Finder *) client_data list_callback : w : call_data];
}
