#include "config.h"
#include "board_view.h"
#include "logical_board.h"
#include "logical_board_ps.h"
#include "logical_board_pc.h"
#include "board_view_ps.h"
#include "game_finder.h"

/*** X Foo ***/

int gXErrorFlag = 0;

int HandleXError (dpy, event)
     Display *dpy;
     XErrorEvent *event;
{
    int a = 0, b = 1;
    b /= a; /* floating point exception! */

#if 0
  gXErrorFlag = 1;

  return 0;
#endif

    return 0;
}


void InstallXErrorHandler (Display *display)
{
  int HandleXError ();

  XSetErrorHandler (HandleXError);
  XFlush (display);
}


void DeInstallXErrorHandler (Display *display)
{
  XSetErrorHandler (NULL);
  XFlush (display);
}


/****************/


int usage (int argc, char *argv[])
{
  fprintf (stderr,
	   "%s [-b] [-s hostname] [-p port]\n"
	   "  -s hostname    Look for a server on machine 'hostname'\n"
	   "  -p port        Look for a server on port 'port'\n"
	   "\n"
	   "  If no server is found, one will be started on localhost:5545\n",
	   argv[ 0 ]);

  return 1;
}


int main (int argc, char *argv[])
{
  XtAppContext app_context;
  Widget top_level;
  Game_Finder *gf;
  Board_View *bv;
  Board_View_PS *bvps;
  char *fallbacks[] = {"*background:grey30",
		       "*XmPushButton*background:grey30",
		       NULL};
  int server_port = 5545;
  char *server_machine = "localhost";
  int listen_port = 6000 + (getpid () % 23000);
  int i;
  char *display=getenv ("DISPLAY");

  srand (time (0));

  if (display==NULL)
    display = ":0";


  top_level = XtInitialize (argv[0], "test3", NULL, 0, &argc, argv);
  app_context = XtDisplayToApplicationContext (XtDisplay (top_level));
  XtAppSetFallbackResources (app_context, fallbacks);


  /* to make editres happy... */
  XtAddEventHandler (top_level, (EventMask)0, True,
		     (XtEventHandler)_XEditResCheckMessages, NULL);


  /* look for interesting arguments */

  for (i=1; i<argc; i++)
    {
      if (strcmp (argv[ i ], "-?") == 0)
	return usage (argc, argv);

      if (strcmp (argv[ i ], "-s") == 0)
	{
	  if (i == argc - 1) return usage (argc, argv);
	  server_machine = argv[ i+1 ];
	}

      if (strcmp (argv[ i ], "-p") == 0)
	{
	  if (i == argc - 1) return usage (argc, argv);
	  server_port = atoi (argv[ i+1 ]);
	}

      if (strcmp (argv[ i ], "-display") == 0)
	{
	  if (i == argc - 1) return usage (argc, argv);
	  display = argv[ i+1 ];
	}
    }


  /* InstallXErrorHandler (XtDisplay (top_level)); */

  XtSetLanguageProc (NULL, NULL, NULL);

  bv = [[Board_View alloc] init_board
			   : argc : argv
			   : app_context
			   : display];

  bvps = [[Board_View_PS alloc] init_ps : listen_port : bv : app_context];

  if (bvps == NULL)
    {
      fprintf (stderr, "bvps is NULL.\n");
      exit (1);
    }


  gf = [[Game_Finder alloc] init
			    : argc : argv
			    : app_context : display
			    : bv : bvps
			    : server_machine
			    : server_port
			    : listen_port];


  [bv set_bvps : (id *) bvps];
  [bv set_gf : (id *) gf];

  XtMainLoop ();
  return 0;
}


