#include "config.h"
#include "netcon.h"
#include "vector.h"
#include "game_info.h"
#include "phaser_chess_ms.h"
#include "enums.h"


static int debug=0;

int main (int argc, char *argv[])
{
  [[[metaserver alloc] init : argc : argv] loop];
  return 0;
}


@implementation metaserver : netcon

- init : (int) argc : (char **) argv
{
  [super init : 5544];
  games = [[Vector alloc] init];

  printf ("local ip addr is: %s\n", [self get_local_ip_addr]);

  return self;
}


- (void) loop
{
  while (1)
    [self catch : -1 : -1];
}


- (void) handle_new_connection : (int) new_channel
{
  printf ("new channel %d\n", new_channel);
}


- (void) handle_gone_connection : (int) channel
{
  printf ("lost channel %d\n", channel);
  /*delete_game_info ((game_info *) [games unset : channel]);*/
  [self report];
}


- (void) process_data : (int) channel : (char *) data : (int) length
{
  char *b = data;
  char *buf;
  char bb[ 2 ];
  int i;
  ms_command pc;

  /* testing */
  printf ("remote address is: %s\n", [self get_remote_ip_addr : channel]);

  [self forget_old_games];

  pc = two_bytes_to_int (b); b += 2;

  if (debug)
    printf ("meta server got %d bytes on channel %d\n", length, channel);

  switch (pc)
    {
    case pc_im_a_server:
      [self extract_and_update : b : channel];
      [self report];
      break;
    case pc_tell_of_games:
      if (channel >= 0)
	{
	  int_to_two_bytes (pc_forget_games, bb);
	  [self throw : channel : bb : 2 : STREAM];
	  for (i=0; i<[games size]; i++)
	    {
	      int len;
	      game_info *gi = (game_info *) [games get : i];
	      if (gi == NULL) continue;
	      buf = encode_game_info (pc_other_game, gi, &len);
	      [self throw : channel : buf : len : STREAM];
	      free (buf);
	    }
	  /* [self close_channel : channel]; */
	}
      break;
    }
}


- (void) forget_old_games
{
  int i;
  time_t now = time (NULL);

  for (i=0; i<[games size]; i++)
    {
      game_info *gi = (game_info *) [games get : i];

      if (gi != NULL)
	if (gi->last_report < now - 120) /* 2 minutes */
	  {
	    delete_game_info ((game_info *) [games unset : i]);
	    [self report];
	  }
    }
}


- (void) extract_and_update : (char *) b : (int) channel
{
  game_info *gi = decode_game_info (b);
  int gi_index = [self find_index_of_game : gi];
  game_info *old_gi = NULL;

  gi->last_report = time (NULL);


  if (debug)
    printf ("writing to log...\n");

  /* write this is a log */
  {
    FILE *fp;
    fp = fopen ("pcms.log", "a");
    if (fp == NULL)
      perror ("fopen");
    else
      {
	fprintf (fp, "%s@%s.%s(%s):%d, %d players\n",
		 gi->uname,
		 gi->host,
		 gi->domain,
		 gi->ip,
		 gi->port,
		 gi->num_players);
	fclose (fp);
      }
  }

  if (debug)
    printf ("done writing to log...\n");

  /* hmmm... I probably shouldn't be advertising yp domain names... */
  /* how do I get the full name of a host? */

  free (gi->domain);
  gi->domain = (char *) malloc (2);
  strcpy (gi->domain, "");

  if (gi_index < 0)
    [games add : (unsigned char *) gi];
  else
    old_gi = (game_info *) [games set : gi_index : (unsigned char *) gi];
  
  if (old_gi)
    delete_game_info (old_gi);

  if (debug)
    printf ("updating board views...\n");

  /* tell everyone that is listening about any changes */
  {
    int i;
    char *buf;
    char bb[ 2 ];

    int_to_two_bytes (pc_forget_games, bb);
    [self throw_to_all : bb : 2 : STREAM];

    for (i=0; i<[games size]; i++)
      {
	int len;
	game_info *gi = (game_info *) [games get : i];
	if (gi == NULL) continue;
	buf = encode_game_info (pc_other_game, gi, &len);
	[self throw_to_all : buf : len : STREAM];
	free (buf);
      }
  }

  if (debug)
    printf ("done updating board views...\n");
}


- (int) find_index_of_game : (game_info *) find_gi
{
  int i;

  for (i=0; i<[games size]; i++)
    {
      game_info *gi = (game_info *) [games get : i];
      if (gi != NULL)
	if ((strcmp (gi->ip, find_gi->ip) == 0) &&
	    (gi->port == find_gi->port))
	  {
	    if (debug)
	      printf ("find_index_of_game found game at index %d\n", i);
	    return i;
	  }
    }
  
  return -1;
}



- (void) report
{
  game_info *gi;
  int i, size;
  size = [games size];


  printf ("-----------------------------------\n");
  for (i=0; i<size; i++)
    {
      gi = (game_info *) [games get : i];
      if (gi == NULL) continue;

      printf ("  %s@%s.%s(%s):%d, %d players\n",
	      gi->uname,
	      gi->host,
	      gi->domain,
	      gi->ip,
	      gi->port,
	      gi->num_players);
    }
  printf ("-----------------------------------\n");

}

@end
